@@ -178,6 +178,8 @@ class AircallConnector(ToucanConnector):
     provided_token: Optional[str]
     auth_flow_id: Optional[str]
     data_source_model: AircallDataSource
+    _oauth_trigger = 'instance'
+    oauth2_version = Field('1', **{'ui.hidden': True})
 
     @staticmethod
     def get_connector_secrets_form() -> ConnectorSecretsForm:
@@ -346,6 +348,7 @@ class AircallConnector(ToucanConnector):
         permissions: Optional[dict] = None,
         offset: int = 0,
         limit: Optional[int] = None,
+        get_row_count: Optional[bool] = False,
     ) -> DataSlice:
         """
         Method to retrieve a part of the data as a pandas dataframe
