@@ -305,7 +305,7 @@ class SnowflakeConnector(ToucanConnector):
             return conn
 
         def alive_function(conn):
-            logger.debug('Check Snowflake connection alive')
+            # logger.debug('Check Snowflake connection alive')
             if hasattr(conn, 'is_closed'):
                 try:
                     return not conn.is_closed()
@@ -334,7 +334,7 @@ class SnowflakeConnector(ToucanConnector):
                     raise TypeError('close is not a function')
 
         connection = snowflake_connection_manager.get(
-            identifier=self.identifier,
+            identifier=self.get_identifier(),
             connect_method=connect_function,
             alive_method=alive_function,
             close_method=close_function,
@@ -393,6 +393,15 @@ class SnowflakeConnector(ToucanConnector):
             )
         return result
 
+    def describe(
+        self,
+        data_source: SnowflakeDataSource,
+    ) -> Dict[str, str]:
+        data_source = self._set_warehouse(data_source)
+        with self._get_connection(data_source.database, data_source.warehouse) as connection:
+            result = SnowflakeCommon().describe(connection, data_source.query)
+        return result
+
     @staticmethod
     def get_snowflake_connection_manager():
         return snowflake_connection_manager
