@@ -5,11 +5,28 @@ from typing import Dict, List, Optional
 
 import pandas as pd
 from pydantic import Field, constr
-from snowflake.connector import DictCursor
+from snowflake.connector import DictCursor, SnowflakeConnection
 
 from toucan_connectors.query_manager import QueryManager
 from toucan_connectors.sql_query_helper import SqlQueryHelper
-from toucan_connectors.toucan_connector import DataSlice, DataStats, ToucanDataSource
+from toucan_connectors.toucan_connector import DataSlice, DataStats, QueryMetadata, ToucanDataSource
+
+type_code_mapping = {
+    0: 'int',
+    1: 'float',
+    2: 'str',
+    3: 'date',
+    4: 'timestamp',
+    5: 'variant',
+    6: 'timestamp_ltz',
+    7: 'timestamp_tz',
+    8: 'timestampe_ntz',
+    9: 'object',
+    10: 'array',
+    11: 'binary',
+    12: 'time',
+    13: 'boolean',
+}
 
 
 class SnowflakeConnectorException(Exception):
@@ -37,12 +54,12 @@ class SnowflakeCommon:
         self.total_returned_rows_count: Optional[int] = -1
         self.execution_time: Optional[float] = None
         self.conversion_time: Optional[float] = None
+        self.column_names_and_types: Optional[Dict[str, str]] = None
 
     def set_data(self, data):
         self.data = data.result()
 
     def set_total_rows_count(self, count):
-        print(count)
         self.total_rows_count = count.result()['TOTAL_ROWS'][0]
 
     def set_total_returned_rows_count(self, count):
@@ -64,7 +81,7 @@ class SnowflakeCommon:
 
     def _execute_query_internal(
         self,
-        connection,
+        connection: SnowflakeConnection,
         query: str,
         query_parameters: Optional[dict] = None,
     ) -> pd.DataFrame:
@@ -106,7 +123,7 @@ class SnowflakeCommon:
 
     def _execute_parallelized_queries(
         self,
-        connection,
+        connection: SnowflakeConnection,
         query: str,
         query_parameters: Optional[Dict] = None,
         offset: Optional[int] = None,
@@ -154,25 +171,36 @@ class SnowflakeCommon:
 
     def fetch_data(
         self,
-        connection,
+        connection: SnowflakeConnection,
         data_source: SfDataSource,
         offset: Optional[int] = None,
         limit: Optional[int] = None,
         get_row_count: bool = False,
     ) -> pd.DataFrame:
+        prepare_query: List = []
+        if data_source.database != connection.database:
+            prepare_query.append(f'USE DATABASE {data_source.database}')
+        if data_source.warehouse != connection.warehouse:
+            prepare_query.append(f'USE WAREHOUSE {data_source.warehouse}')
+        if len(prepare_query) > 0:
+            self._execute_query(connection, ';'.join(prepare_query) + ';')
+            self.logger.info(
+                f'Connection changed to use database {connection.database} and warehouse {connection.warehouse}'
+            )
+
         ds = self._execute_parallelized_queries(
             connection, data_source.query, data_source.parameters, offset, limit, get_row_count
         )
         return ds.df
 
     def retrieve_data(
-        self, c, data_source: SfDataSource, get_row_count: bool = None
+        self, connection: SnowflakeConnection, data_source: SfDataSource, get_row_count: bool = None
     ) -> pd.DataFrame:
-        return self.fetch_data(c, data_source, get_row_count=get_row_count)
+        return self.fetch_data(connection, data_source, get_row_count=get_row_count)
 
     def get_slice(
         self,
-        connection,
+        connection: SnowflakeConnection,
         data_source: SfDataSource,
         offset: Optional[int] = None,
         limit: Optional[int] = None,
@@ -189,6 +217,7 @@ class SnowflakeCommon:
         )
         return DataSlice(
             df=result,
+            query_metadata=QueryMetadata(columns=self.column_names_and_types),
             # In the case of user defined limit/offset, get the info
             # Not used for now
             # input_parameters={
@@ -211,3 +240,35 @@ class SnowflakeCommon:
             query = f"{query} LIKE '{database_name}'"
         res = self._execute_query(connection, query).to_dict().get('name')
         return [database for database in res.values()] if res else []
+
+    def describe(self, connection, query):
+        return QueryManager().describe(
+            describe_method=self._describe,
+            connection=connection,
+            query=query,
+        )
+
+    def _describe(
+        self,
+        connection: SnowflakeConnection,
+        query: str,
+    ) -> Dict[str, str]:
+        description_start = timer()
+        cursor = connection.cursor(DictCursor)
+        describe_res = cursor.describe(query)
+        description_end = timer()
+        description_time = description_end - description_start
+        self.logger.info(
+            f'[benchmark][snowflake] - description {description_time} seconds',
+            extra={
+                'benchmark': {
+                    'operation': 'describe',
+                    'execution_time': description_time,
+                    'connector': 'snowflake',
+                    'query': query,
+                    'result': describe_res,
+                }
+            },
+        )
+        res = {r.name: type_code_mapping.get(r.type_code) for r in describe_res}
+        return res
